#include <iostream>
#include <math.h>
#include <stdlib.h>
#include <GL/glut.h>

#include "gfxio.h"

#include "RvSpace.h"
#include "RvCamera.h"
#include "RvModel.h"
#include "RvInit.h"

#define XS 1300
#define YS  1200
char IMGBUF [XS*YS*3];
char name [256];
int snapcnt =0;

#define NB_MY_WINDOWS 1

//RvGridSpace *WORLD;
RvSpace *WORLD;
RvViewer *viewer;
RvModel model;

/**************************************** myGlutKeyboard() **********/

void myGlutKeyboard(unsigned char Key, int x, int y)
{
  switch(Key)
  {
  case 27:
  case 'q':
    exit(0);
    break;

	case 's':
	case 'S':
		int x,y;
		viewer->getWindowSize(x,y);
		viewer->readFrameBuffer(IMGBUF);

		sprintf(name,"snapshot%d.tif",snapcnt);
		saveTiffRGB(name,x,y,IMGBUF,GFXIO_FLIP_VERTICAL);
		std::cerr << "done snapshot" << std::endl;
		snapcnt++;
	break;

  case 'v':
  case 'V' :
	viewer->recallCamera();

	break;
  case 'p':
	viewer->toggleRayPick();
	break;
  }

}

/***************************************** myGlutIdle() ***********/

void myGlutIdle( void )
{
  glutPostRedisplay();

}

/***************************************** myGlutMouse() **********/

void myGlutMouse(int button, int button_state, int x, int y )
{
	viewer->processMouseEvents(button,button_state,glutGetModifiers(),x,y);
}

/***************************************** myGlutMotion() **********/

void myGlutMotion(int x, int y )
{
  viewer->update(x,y);

}

/**************************************** myGlutReshape() *************/

void myGlutReshape( int x, int y )
{
  glViewport( 0, 0, x, y );

  viewer->resize(x,y);

}

/***************************************** myGlutDisplay() *****************/

void myGlutDisplay( void )
{

  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  glEnable( GL_LIGHTING );
  glEnable (GL_DEPTH_TEST);
  glEnable (GL_TEXTURE_2D);
  glEnable(GL_CULL_FACE);

  viewer->getCamera().loadGL();

  /*
   * Set various uniform parameters including the ModelViewProjection
   * matrix for transforming the incoming position into HPOS.
   */
  RvVec3 eye = viewer->getCamera().getPosition();
  //glColor3fv(&model.materials[0].diffuse[0]);

  //model.render();
  viewer->drawScene();

  viewer->drawAxes();

  char FPS [256];
  sprintf(FPS,"%d FPS",(int)viewer->getFrameRate());
  RvDrawText(10,10,FPS);
 
  viewer->refreshView(30);

  glutSwapBuffers();
}



int main(int argc,char **argv)
{
  if( argc != 2 ) {
    std::cerr << "Usage: BasicViewer filename.obj" << std::endl;
    exit(1);
  }

  RvInit();

  glutInitWindowSize( 640, 480 );
  glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
  glutCreateWindow( "Test Rv" );

  glutDisplayFunc( myGlutDisplay );
  glutReshapeFunc( myGlutReshape );
  glutKeyboardFunc( myGlutKeyboard );
  glutMotionFunc( myGlutMotion );
  glutMouseFunc( myGlutMouse );
  glutIdleFunc( myGlutIdle );

  GLfloat glfLightAmbient[]  = { 0.2f, 0.2f, 0.2f, 1.0f };
  GLfloat glfLightDiffuse[]  = { 0.9f, 0.9f, 0.9f, 1.0f };
  GLfloat glfLightSpecular[] = { 0.9f, 0.9f, 0.9f, 1.0f };
  GLfloat lightPosition[]    = { -100.0f, 100.0f, 100.0f, 1.0f };

  glLightfv ( GL_LIGHT0, GL_AMBIENT,  glfLightAmbient  );
  glLightfv ( GL_LIGHT0, GL_DIFFUSE,  glfLightDiffuse  );
  glLightfv ( GL_LIGHT0, GL_SPECULAR, glfLightSpecular );
  glLightfv ( GL_LIGHT0, GL_POSITION, lightPosition);
  glEnable  ( GL_LIGHT0 );
  glEnable  ( GL_LIGHTING );

  glEnable( GL_DEPTH_TEST );

  glClearColor( 0.8f, 0.8f, 0.8f, 0.0f );

  if( model.loadObjMtl(argv[1]) ) {
	  std::cerr << "model successfully loaded " << std::endl;
  }
  else {
	  std::cerr << "Cant load model " << argv[1] << ". exiting " << std::endl;
	  exit(1);
  }
  model.init();

  std::cout << "bbox " << model.boundingBox.min << " " << model.boundingBox.max << std::endl;

  WORLD  = new RvPrimitiveList();
  WORLD->bbox =model.boundingBox;
  WORLD->getPrimitiveList().insert(&model);

  viewer = new RvExaminerViewer;
  viewer->setScene( WORLD );
  viewer->getCamera().lookAt( WORLD->bbox );
  viewer->saveCamera();
  viewer->resize(640,480);

  glutMainLoop();

  return(0);
}
