#include "RvCamera.h"
#include "RvSpace.h"
#include "RvSys.h"
#include <assert.h>
#include <stdio.h>

#include <GL/glut.h>

void RvCamera::save(char *fname)
{
	FILE *F = fopen(fname,"w");

	fprintf(F,"#RV camera file V.1.0\n");

	RvVec P= position;
	fprintf(F,"%f %f %f\n", P[0],P[1],P[2]);
	
	// Also save orientation
	//....
	//
	fprintf(F,"%f %f %f %f %f %f\n",
	 	frust.nearDist,
	 	frust.farDist,
	 	frust.fovAngle,
	 	frust.aspectRatio,
	 	frust.width,
	 	frust.height);

	fclose(F);	
}

void RvCamera::load(char *fname)
{
	FILE *F = fopen(fname,"r");

	fscanf(F,"#RV camera file V.1.0\n");

	float P[3];
	float Q[4];
	fscanf(F,"%f %f %f\n", &P[0],&P[1],&P[2]);

	fscanf(F,"%f %f %f %f\n", &Q[0],&Q[1],&Q[2],&Q[3]);

	position = RvVec(P);
	// Also load orientation
	// ....
	
	float D[6];
	fscanf(F,"%f %f %f %f %f %f\n",&D[0],&D[1],&D[2],&D[3],&D[4],&D[5]);
	
 	frust.nearDist = D[0];
 	frust.farDist = D[1];
 	frust.fovAngle = D[2];
 	frust.aspectRatio =  D[3];
 	frust.width = D[4];
 	frust.height = D[5];

	fclose(F);	
}


void RvCamera::pointAt(const RvVec &target)
{
	RvVec t,u;
	getOrientation(t,u);
	RvVec dir = (target - position);
	dir.normalize();
	RvQuat Q = RvQuat::fromBallPoints(t,dir);
	RvVec p = position;
	position = 0.0;
	rotate(Q);
	setPosition(p);

	frust.updateVPort();
}

void RvCamera::draw()
{
        glDisable(GL_LIGHTING);
	glColor3f(1,1,0);
        glBegin(GL_LINES);

	RvVec V = position;
	RvVec v1(-10,-10,-10*frust.nearDist);
	RvVec v2(-10,10,-10*frust.nearDist);
	RvVec v3(10,10,-10*frust.nearDist);
	RvVec v4(10,-10,-10*frust.nearDist);

	// Need to get orientation from camera orientation
	// ....
	RvQuat orient(0,0,1,0);

	RvVec r1 = V+RvQuat::apply(v1,orient);
	RvVec r2 = V+RvQuat::apply(v2,orient);
	RvVec r3 = V+RvQuat::apply(v3,orient);
	RvVec r4 = V+RvQuat::apply(v4,orient);

	glVertex3f(V[0],V[1],V[2]);
	glVertex3f(r1[0],r1[1],r1[2]);
	glVertex3f(V[0],V[1],V[2]);
	glVertex3f(r2[0],r2[1],r2[2]);
	glVertex3f(V[0],V[1],V[2]);
	glVertex3f(r3[0],r3[1],r3[2]);
	glVertex3f(V[0],V[1],V[2]);
	glVertex3f(r4[0],r4[1],r4[2]);
	
	glVertex3f(r1[0],r1[1],r1[2]);
	glVertex3f(r2[0],r2[1],r2[2]);
	glVertex3f(r2[0],r2[1],r2[2]);
	glVertex3f(r3[0],r3[1],r3[2]);
	glVertex3f(r3[0],r3[1],r3[2]);
	glVertex3f(r4[0],r4[1],r4[2]);
	glVertex3f(r4[0],r4[1],r4[2]);
	glVertex3f(r1[0],r1[1],r1[2]);

	glEnd();
		
}

void RvCamera::lookAt(const RvBox &target)
{
	std::cerr << "Lookat set :" <<
	target.min << "/ " << target.max << std::endl;
	std::cerr << "Lookat center :" <<
	target.center() << std::endl;
	std::cerr << "Lookat radius :" <<
	target.radius() << std::endl;

	position = target.center() + 4*RvVec(0,0,1)*target.radius();

	float a = 
			(target.center() -position).length()
			- target.radius();

		if (a<=0.1) a=0.1;
		frust.nearDist = a;
		float b =
		frust.farDist  = a+2*target.radius()+1;

	std::cerr << "Frustum near/far distance:" << frust.nearDist << " " << frust.farDist << std::endl;
	std::cerr << "Camera position: " << position << std::endl;

	float dist= (position-target.center()).length();
	float radtheta = 2* atan2(target.radius(), dist);
	frust.fovAngle = radtheta;

	frust.updateVPort();

	std::cerr << "Camera FOV " << frust.fovAngle << std::endl;
}

void RvCamera::loadGL()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (mode == RV_PERSPECTIVE)
	{
		gluPerspective(RV_RAD2DEG(frust.fovAngle),
		frust.aspectRatio,frust.nearDist, frust.farDist);
	}
	else if (mode == RV_ORTHOGRAPHIC)
	{
  		glOrtho( -frust.width/2.0, frust.width/2.0, 
		-frust.height/2.0, frust.height/2.0,
		frust.nearDist,frust.farDist );
	}
	else assert(0!=0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// Add code here to take into account 
	// camera orientation
	// ....

	glTranslatef(-position[0],-position[1],-position[2]);
}


void RvCamera::getGLMatrix(float *matrix)
{

}

///////////////////////////////////////////////////

void RvViewer::setupLeftView()
{
	glViewport(0,h/2+20,w, h/2-20);
	RvCamera leftcam = cam;
	RvVec v = leftcam.getPosition();
	RvVec one(1,0,0);
	one.normalize();
	leftcam.setPosition(v-0.2*one);
	leftcam.loadGL();
}

void RvViewer::setupRightView()
{
	glViewport(0,0,w,h/2-20);
	RvCamera rightcam = cam;
	RvVec v = rightcam.getPosition();
	RvVec one(1,0,0);
	one.normalize();
	rightcam.setPosition(v+0.2*one);
	rightcam.loadGL();

}


///////////////////////////////////////////////////

void RvViewer::drawAxes()
{
	float a[4];
	glGetFloatv(GL_VIEWPORT,a);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(20,1.33,2,10);

  	glMatrixMode( GL_MODELVIEW );
  	glLoadIdentity();
	glLineWidth(3.0);

	glTranslatef(0,0,-4);

	// Add code for orientation here
	// ....

  	glDisable( GL_LIGHTING );  
  	glViewport(w-50,10,40,40);

	glEnable(GL_DEPTH_TEST);
	//glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_TEXTURE_1D);

	glBegin(GL_LINES);

	glColor3f(1,0,0);
	glVertex3f(0,0,0);
	glVertex3f(1,0,0);

	glColor3f(0,1,0);
	glVertex3f(0,0,0);
	glVertex3f(0,1,0);

	glColor3f(0,0,1);
	glVertex3f(0,0,0);
	glVertex3f(0,0,1);

	glEnd();

	glViewport(a[0],a[1],a[2],a[3]);
}

//
// Convert from window coordinates to normalized x/y 
// coordinates
//
// initialize local coordinate system
//
void RvViewer::update(int mousex,int mousey)
{
	// Call rotate/translate/zoom as appropriate
	//....
}

void RvViewer::refreshView(int targetFPS)
{
	fpsctl.enforce(targetFPS);
}

RvModel *RvViewer::getPrimitive(int mousex,int mousey,
RvVec *P, RvVec *N, float *D) 
{ 
	// creer le rayon en utilisant getRay
	// et intersect le RvSpace space avec
	// ....
	return NULL;
}

void RvViewer::drawScene()
{
	space->draw();	
}

void RvViewer::enableFullScreen() 
{
	if (full) return;
	oldw = w; oldh = h;	
	full = 1;
	glutFullScreen();
}
void RvViewer::disableFullScreen() 
{
	if (!full) return;
 full=0; glutReshapeWindow(oldw,oldh);	
}

void RvViewer::toggleFullScreen()
{
	if (full) disableFullScreen();
	else enableFullScreen();
}

void RvViewer::readFrameBuffer(char *IMGBUF)
{
	glPixelStorei(GL_PACK_ALIGNMENT,1);
	glReadBuffer(GL_FRONT);
	glReadPixels(0,0,w,h,GL_RGB,GL_UNSIGNED_BYTE,IMGBUF);
}


void RvViewer::processMouseEvents(
int button,int button_state, int modifiers, int x,int y)
{
 	dV = 0.0;
	startx = 2.0*x/w-1.0; 
	starty = 2.0*y/h-1.0;
	starty *=-1;
	cam.getOrientation(T,U);	
	W = T.cross(U);	
	W.normalize();
}

///////////////////////////////////////////////////


void RvExaminerViewer::processMouseEvents(
int button,int button_state, int modifiers, int x,int y)
{
	RvViewer::processMouseEvents(button,button_state,modifiers,x,y);

	// Fill in the treatment of mouse events here
	// (ie call begin translate/rotate etc)
	//....
}

///////////////////////////////////////////////////

void RvWalkViewer::processMouseEvents(
int button,int button_state, int modifiers, int x,int y)
{
RvViewer::processMouseEvents(button,button_state,modifiers,x,y);
	// Fill in the treatment of mouse events here
	// (ie call begin translate/rotate etc)
	//....
}

///////////////////////////////////////////////////

void RvFrameRateCtlStruct::enforce(int target)
{
	RV_NB_CUR++;
	double T = RvGetSysTime();
	double A = 1.0/(T-RV_OLDT);
	A = 1.0/target - 1.0/A;
	RV_OLDT = T;
	if (fabs(A)>fabs(RV_OLDA))
		RV_DT = RV_ODT;
	else 
		RV_DT += A;
	RV_OLDA = A;
	RV_ODT = RV_DT;
	if (T-RV_OLDT2>RV_FREQ)
	{
		RV_FRAME_RATE = 1.0/((T-RV_OLDT2)/RV_NB_CUR);
		RV_OLDT2 = T;
		RV_NB_CUR = 0;	
	}
	//std::cerr << (int) (1000*RV_DT) << std::endl;
	if (target) RvSleep(RV_MAX(0,1000.0*RV_DT));
}

void RvFrameRateCtlStruct::init(float FREQ)
{
	RV_NB_CUR=0;
	RV_OLDT= RV_OLDT2 = RvGetSysTime();
	RV_FRAME_RATE =0.0;
	RV_DT=0.0;
	RV_FREQ = FREQ;
	std::cerr << "frame rate avg every " << RV_FREQ << " sec." << std::endl;
}

///////////////////////////////////////////////////

void RvDrawText(int x,int y,char *txt, unsigned long rgba)
{
  glDisable( GL_LIGHTING );  /* Disable lighting while we render text */
  glDisable( GL_TEXTURE_2D );  /* Disable tex while we render text */
  glDisable( GL_TEXTURE_1D );  /* Disable tex while we render text */
  glDisable( GL_DEPTH_TEST );  /* Disable z-buff while we render text */
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
unsigned char *RGBA = (unsigned char *) (&rgba);
glColor4ubv(RGBA);
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluOrtho2D( 0.0, 100.0, 0.0, 100.0  );
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  glRasterPos2i( x, y );
  glDisable(GL_DEPTH_TEST);

  for(int i=0;i<strlen(txt);i++)
	glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,txt[i]);
}

void RvDrawCylinder(const RvVec &c1,const RvVec &n,float r,
unsigned long rgba,int n1,int n2)
{
}


void RvGLCylinder(float r, float h,int n1,int n2)
{
	RvVec POINTS [1024][20];
	int i;
	
	for( i=0;i<n2;i++)
		for(int j=0;j<=n1;j++)
		{
			POINTS[j][i]= 
				RvVec(r*cos(j*RV_M_2PI/n1),
					r*sin(j*RV_M_2PI/n1),
					-i*h/(n2-1));
		}

	for(i=0;i<n2-1;i++)
	{
		glBegin(GL_TRIANGLE_STRIP);
		//for(int j=n1-1;j>=0;j--)
		for(int j=0;j<=n1;j++)
		{
			glNormal3f(cos(j*RV_M_2PI/n1),
				   sin(j*RV_M_2PI/n1),0);
			glVertex3f( POINTS[j][i][0],
					POINTS[j][i][1],
					POINTS[j][i][2]);
			glVertex3f( POINTS[j][i+1][0],
					POINTS[j][i+1][1],
					POINTS[j][i+1][2]);
		}
		glEnd();
	}

}


void RvGLCone(float r, float h, int n1,int n2)
{
	RvVec POINTS [1024][20];
	int i;

	for( i=1;i<=n2;i++)
		for(int j=0;j<=n1;j++)
		{
			POINTS[j][i-1]= 
				RvVec(i*r*cos(j*RV_M_2PI/n1)/n2,
					i*r*sin(j*RV_M_2PI/n1)/n2,0);
		}

	glBegin(GL_TRIANGLE_FAN);
	glVertex3f(0,0,h);
	for(i=0;i<=n1;i++)
	{
		RvVec v = (POINTS[(i+1)%n1][n2-1]-POINTS[i][n2-1]).
		cross(POINTS[i][n2-1]-RvVec(0,0,h));
		v.normalize();
		glNormal3f(-v[0],-v[1],-v[2]);
		glVertex3f( POINTS[i][n2-1][0],
				POINTS[i][n2-1][1],
				POINTS[i][n2-1][2]);
	}
	glEnd();
}

void RvGLCircle(float r,int n)
{
	RvVec3 POINTS [1024];
	int j;
	for( j=0;j<=n;j++)
		POINTS[j] = RvVec3(r*cos(j*RV_M_2PI/n),
				r*sin(j*RV_M_2PI/n),0);

	glBegin(GL_LINE_STRIP);
	for( j=0;j<=n;j++)
		glVertex3fv( &POINTS[j][0]);
	glEnd();
}

void RvGLDisc(float r,int n1,int n2)
{
	RvVec POINTS [1024][20];
	int i;
	
	for( i=1;i<=n2;i++)
		for(int j=0;j<=n1;j++)
		{
			POINTS[j][i-1]= 
				RvVec(i*r*cos(j*RV_M_2PI/n1)/n2,
					i*r*sin(j*RV_M_2PI/n1)/n2,0);
		}

	glBegin(GL_TRIANGLE_FAN);
	glNormal3f(0,0,-1);
	glVertex3f(0,0,0);
	for(i=n1;i>=0;i--)
		glVertex3f( POINTS[i][n2-1][0],
				POINTS[i][n2-1][1],
				POINTS[i][n2-1][2]);
	glEnd();
}

void	RvDrawCircle(const RvVec &c,const RvVec &n,float r,
		unsigned long rgba,int n1)
{
  	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();

	unsigned char *RGBA = (unsigned char *) (&rgba);
	glColor4ubv(RGBA);

	float m[4*4];
	RvVec VV = n; VV.normalize();
	RvQuat Q = RvQuat::fromBallPoints(RvVec(0,0,-1),n);
	Q.getRotationMatrix(m);

	glTranslatef(c[0],c[1],c[2]);
	glMultMatrixf(m);
	glScalef(r,r,0);

	RvGLCircle(1,n1);

	glPopMatrix();

}

void RvDrawDisc(const RvVec &c,const RvVec &n,float r,
unsigned long rgba,int n1,int n2)
{
  	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();

	unsigned char *RGBA = (unsigned char *) (&rgba);
	glColor4ubv(RGBA);

	float m[4*4];
	RvVec VV = n; VV.normalize();
	RvQuat Q = RvQuat::fromBallPoints(RvVec(0,0,-1),n);
	Q.getRotationMatrix(m);

  	glTranslatef(c[0],c[1],c[2]);
	glMultMatrixf(m);
	glScalef(r,r,0);

	RvGLDisc(1,n1,n2);

	glPopMatrix();
}

void RvDraw3DArrow(const RvVec &origin,const RvVec &vec,
unsigned long rgba,float scale,int n1,int n2)
{
  	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();

	float m[4*4];
	RvVec VV = vec; VV.normalize();
	RvQuat Q = RvQuat::fromBallPoints(RvVec(0,0,1),VV);
	Q.getRotationMatrix(m);

  	glTranslatef(origin[0],origin[1],origin[2]);
	glScalef(scale,scale,scale);
	glMultMatrixf(m);

	unsigned char *RGBA = (unsigned char *) (&rgba);
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glColor4ubv(RGBA);

	glEnable(GL_LIGHTING);
  	glTranslatef(0,0,-2);
	RvGLCone(1,2,n1,n2);
	RvGLDisc(1,n1,n2);
	RvGLCylinder(0.5,2,n1,n2);
  	glTranslatef(0,0,-2);
	RvGLDisc(0.5,n1,n2);

	glPopMatrix();
}

void RvDraw3DText(RvVec &pos,char *text,int rgba,float scale)
{
	glDisable(GL_LIGHTING);
	unsigned char *RGBA = (unsigned char *) (&rgba);
	glColor4ubv(RGBA);

	float modelmat [16];
	glGetFloatv(GL_MODELVIEW_MATRIX,modelmat);

	RvQuat q = RvQuat::fromMatrixGL(modelmat);
	q.conj();

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

  	RvVec axis;
  	float   angle;
  	q.toAxisAngle( &angle, &axis );

	glTranslatef(pos[0],pos[1],pos[2]);
  	glRotatef( angle, axis[ 0 ], axis[ 1 ], axis[ 2 ] );
	glScalef(scale,scale,scale);
 	for(int i=0;i<strlen(text);i++)
		glutStrokeCharacter(GLUT_STROKE_ROMAN,text[i]);
	glPopMatrix();

}

