#ifndef RV_GEOM_H
#define RV_GEOM_H

#include "RvVec.h"
#include "RvDefs.h"

struct RvRay
{
        RvVec origin;
        RvVec dir;
        float L;
        float E;
        int depth;
	int type;
	void *userData;
};

//////////////////////////////////////////////////////////////////

struct RvBox
{
	RvBox() { min = max = 0.0;}
	RvBox(const RvVec &_min, const RvVec &_max)
	{
		min = _min;
		max = _max;
	}
  RvBox( const RvVec3 *aVector, unsigned nVector ) {

    min[ 0 ] = aVector[ 0 ][ 0 ];
    max[ 0 ] = aVector[ 0 ][ 0 ];
    min[ 1 ] = aVector[ 0 ][ 1 ];
    max[ 1 ] = aVector[ 0 ][ 1 ];
    min[ 2 ] = aVector[ 0 ][ 2 ];
    max[ 2 ] = aVector[ 0 ][ 2 ];

    // find the x,y,z min and max of the mesh
    for ( unsigned i = 1 ; i < nVector ; i++ ) {
      min[ 0 ] = RV_MIN( min[ 0 ], aVector[ i ][ 0 ] );
      max[ 0 ] = RV_MAX( max[ 0 ], aVector[ i ][ 0 ] );
      min[ 1 ] = RV_MIN( min[ 1 ], aVector[ i ][ 1 ] );
      max[ 1 ] = RV_MAX( max[ 1 ], aVector[ i ][ 1 ] );
      min[ 2 ] = RV_MIN( min[ 2 ], aVector[ i ][ 2 ] );
      max[ 2 ] = RV_MAX( max[ 2 ], aVector[ i ][ 2 ] );
    }
  }

	void draw(unsigned int RGBA = RV_RGBA2INT(1,0,0,1));

	void scale(const RvVec &s)
	{
		min *=s; max*=s;
	}
	RvVec center() const
	{
		return ((min+max)/2.0);
	}
	float diagonal() const
	{
		return (max-min).length();

	}
	float radius() const
	{
		return diagonal()/2.0;
	}

	int contains(const RvVec &v) const
	{
		if ((v[0]>=min[0] && v[0]<=max[0])
		 &&(v[1]>=min[1] && v[1]<=max[1])
		 &&(v[2]>=min[2] && v[2]<=max[2]))
			return 1;
		else return 0;
	}
	bool intersect( const RvRay &r, float &t1, float &t2 ) const;
	int intersect(const RvRay &r,RvVec &pos,RvVec &norm,float &d,int backf_cull=1);
	float distance(const RvVec &v);

	RvVec min;
	RvVec max;

};

#endif
