///////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////
#ifndef __RV_MESH_H__
#define __RV_MESH_H__

#include "RvVec.h"
#include "RvQuat.h"
#include "RvGeom.h"
#include "RvDisplayMaterial.h"

#include <GL/glut.h>
#include <iostream>
#include <stdlib.h>

class PrimitiveGroup {
public:
  enum PrimType {
	  PT_STRIP = GL_TRIANGLE_STRIP,
	  PT_FAN   = GL_TRIANGLE_FAN,
    PT_TRI   = GL_TRIANGLES,
    PT_POINT = GL_POINTS
  };

  PrimType type;
	unsigned int numIndices;
	unsigned short* indices;

  ////////////////////////////////////////////////////////////////////////////////////////

	PrimitiveGroup() : type( PT_STRIP ), numIndices( 0 ), indices( NULL ) {}
	~PrimitiveGroup()	{
    if( indices )
			delete[] indices;
	}

  bool convertToTriangles() {
    if ( ( indices == NULL ) || ( numIndices < 3 ) )
      return false;

    unsigned i, j, nTriangles;
    unsigned short *tmp;

    switch( type ) {
      case PT_TRI:
        break;

      case PT_STRIP :
        nTriangles = numIndices - 2;
        numIndices = nTriangles * 3;
        tmp        = new unsigned short[ numIndices ];

        j = 0;
        for ( i = 0; i < nTriangles; i++ ) {
          tmp[ j++ ] = indices[ i ];
          tmp[ j++ ] = indices[ i + 1 ];
          tmp[ j++ ] = indices[ i + 2 ];
        }
        delete [] indices;

        indices = tmp;
        type    = PT_TRI;
        break;

      default:
        return false;
    }
  };

    bool shuffle() {
      if ( type == PT_TRI ) {
        unsigned nRand = numIndices;
        unsigned nTri  = nRand / 3;
        unsigned short tmp;
        unsigned rnd1, rnd2;

        for ( unsigned i = 0; i < nRand; i++ ) {
          rnd1 = rand()%nTri * 3;
          rnd2 = rand()%nTri * 3;

          tmp = indices[ rnd1 ];
          indices[ rnd1 ] = indices[ rnd2 ];
          indices[ rnd2 ] = tmp;
          rnd1++;
          rnd2++;
          tmp = indices[ rnd1 ];
          indices[ rnd1 ] = indices[ rnd2 ];
          indices[ rnd2 ] = tmp;
          rnd1++;
          rnd2++;
          tmp = indices[ rnd1 ];
          indices[ rnd1 ] = indices[ rnd2 ];
          indices[ rnd2 ] = tmp;
        }
        return true;
      }
      return false;
    };


};

struct Vertex {
  RvVec2 texCoord;
  RvVec3 normal;
  RvVec3 vertex;
};

class RvMesh {
  /////////////////////////////////////////////////////////////////////////////

public:
  /////////////////////////////////////////////////////////////////////////////
  Vertex              *vertex;
  RvDisplayMaterial *material;
  PrimitiveGroup      *primitiveGroup;
  unsigned nPrimitiveGroup;
  unsigned nVertex;

  /////////////////////////////////////////////////////////////////////////////
	RvMesh();
	~RvMesh();

  /////////////////////////////////////////////////////////////////////////////
  bool init();
  bool render();

  /////////////////////////////////////////////////////////////////////////////
};

#endif
