#include "RvModel.h"
#include "RvMesh.h"
#include <GL/glut.h>
#include <string.h>

///////////////////////////////////////////////////////////////////////////////
RvModel::RvModel() : meshes( NULL ),materials( NULL ) {
  orientation.fromEuler( 0,0,0);
  scale = 1.0;
  position=0.0;
}

RvModel::~RvModel() {
  if ( meshes )
    delete [] meshes;
  if ( materials )
    delete [] materials;
}

#include "../ObjParser/objMtlParser.h"

bool RvModel::loadObjMtl( char* filename ) {

  bool rep=true;

  if(!objMtlReadModelFile(filename, rep))
	  return false;

  int n;
  objMtlSimpleMaterial* matTab = objMtlGetMaterialTable(n);
  nMaterial = (unsigned int) n;
  materials = new RvDisplayMaterial[ nMaterial ];
  int i;
  for( i = 0 ; i < nMaterial ; i++ ) {
    materials[ i ].ambient[0] = matTab[i].Ka[0];
    materials[ i ].ambient[1] = matTab[i].Ka[1];
    materials[ i ].ambient[2] = matTab[i].Ka[2];

    materials[ i ].specular[0] = matTab[i].Ks[0];
    materials[ i ].specular[1] = matTab[i].Ks[1];
    materials[ i ].specular[2] = matTab[i].Ks[2];

    materials[ i ].diffuse[0] = matTab[i].Kd[0];
    materials[ i ].diffuse[1] = matTab[i].Kd[1];
    materials[ i ].diffuse[2] = matTab[i].Kd[2];

    materials[ i ].alpha = materials[ i ].shininess = 1.;
    // Read texture name and initialize appropriate fields
    // ....
    if( matTab[ i ].texture_name ) {
	    //....
    }
  }

  nMesh = 1;
  nMesh = objMtlGetNumMeshes();

  meshes = new RvMesh[nMesh];

  unsigned short nPolys;
  for ( i = 0; i < nMesh; i++ ) {
    nPolys = objMtlNumPolys();
    meshes[ i ].primitiveGroup = new PrimitiveGroup[ 1 ];
    meshes[ i ].nPrimitiveGroup = 1;
    // set number of indices and indices themselves
    int mid;
    objMtlGetIndices(i, meshes[ i ].primitiveGroup[ 0 ].numIndices,
    meshes[ i ].primitiveGroup[ 0 ].indices, mid);

    meshes[ i ].material = &materials[ mid ];
    meshes[ i ].primitiveGroup[ 0 ].type = PrimitiveGroup::PT_TRI;

    objMtlSimpleVertex *tmp;
    objMtlGetVertices( meshes[ i ].nVertex, tmp);

    meshes[ i ].vertex = new Vertex[ meshes[i].nVertex ];
    for( int j = 0; j < meshes[i].nVertex; j++ ) {
	meshes[i].vertex[j].vertex[0] = tmp[j].xyz[0];
	meshes[i].vertex[j].vertex[1] = tmp[j].xyz[1];
	meshes[i].vertex[j].vertex[2] = tmp[j].xyz[2];
	
	meshes[i].vertex[j].normal[0] = tmp[j].n[0];
	meshes[i].vertex[j].normal[1] = tmp[j].n[1];
	meshes[i].vertex[j].normal[2] = tmp[j].n[2];

	meshes[i].vertex[j].texCoord[0] = tmp[j].uvw[0];
	meshes[i].vertex[j].texCoord[1] = tmp[j].uvw[1];

	// update bounding box
	if( boundingBox.min[0] > tmp[j].xyz[0] )
		boundingBox.min[0] = tmp[j].xyz[0];
	if( boundingBox.min[1] > tmp[j].xyz[1] )
		boundingBox.min[1] = tmp[j].xyz[1];
	if( boundingBox.min[2] > tmp[j].xyz[2] )
		boundingBox.min[2] = tmp[j].xyz[2];

	if( boundingBox.max[0] < tmp[j].xyz[0] )
		boundingBox.max[0] = tmp[j].xyz[0];
	if( boundingBox.max[1] < tmp[j].xyz[1] )
		boundingBox.max[1] = tmp[j].xyz[1];
	if( boundingBox.max[2] < tmp[j].xyz[2] )
		boundingBox.max[2] = tmp[j].xyz[2];

    }
    delete tmp;
  }
  for ( i = 0; i < nMaterial; i++ )
    materials[ i ].load();
  return true;
}


///////////////////////////////////////////////////////////////////////////////
bool RvModel::load( char* filename ) {
  return true;
}

///////////////////////////////////////////////////////////////////////////////
bool RvModel::init() {
for ( unsigned i = 0; i < nMaterial; i++ ) {
	materials[i].initGL();
}
  return true;
}

///////////////////////////////////////////////////////////////////////////////
bool RvModel::render() {
  glMatrixMode( GL_MODELVIEW );
  glPushMatrix();
  set();

  for ( unsigned i = 0; i < nMesh; i++ ) {
    meshes[ i ].init();
    meshes[ i ].render();
  }

  glPopMatrix();
  return true;
}

///////////////////////////////////////////////////////////////////////////////
bool RvModel::set() {

  RvVec axis;
  float   angle;

  orientation.toAxisAngle( &angle, &axis );

  //cerr << position << "  " << orientation << " " << scale << endl;

  glTranslatef( position[ 0 ], position[ 1 ], position[ 2 ] );
  glRotatef( angle, axis[ 0 ], axis[ 1 ], axis[ 2 ] );
  glScalef(scale[0],scale[1],scale[2]);

  return true;
}

///////////////////////////////////////////////////////////////////////////////
void RvModel::setOrientation( const RvQuat &orientation ) {
  this->orientation = orientation;
}

///////////////////////////////////////////////////////////////////////////////
void RvModel::setPosition( const RvVec3 &position ) {
  this->position = position;
}
///////////////////////////////////////////////////////////////////////////////
void RvModel::setScale( const RvVec3 &scale ) {
  this->scale = scale;
}
