///////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __RV_MODEL_H__
#define __RV_MODEL_H__

#include "RvDefs.h"
#include "RvVec.h"
#include "RvQuat.h"
#include "RvGeom.h"

#include "RvMesh.h"

#include <iostream>
#include <stdio.h>


class RvModel {
protected:
  
  //////////////////////////////////////////////////////////////////////////////
  // a set of mesh
  RvMesh     *meshes;
  // orientation and position of the model in world space
  RvQuat orientation;
  RvVec3 position;
  RvVec3 scale;
public:
  // num of mesh
  unsigned nMesh;
  // a set of all material of the model
  unsigned nMaterial;
  RvDisplayMaterial *materials;
  // bounding box of the model
  RvBox boundingBox;
  // constructor / destructor
  RvModel();
	~RvModel();

  // load sup model return false if an error occur
  bool load( char* filename );

  // load .obj/.mtl model
  bool loadObjMtl(char*);

  // init model
  bool init();
  // render model
  bool render();
  // apply orienation and position to openGL Modelview
  bool set();
  // set orientation and position of the model in world space
  void setPosition( const RvVec3 &position );
  void setOrientation( const RvQuat &orientation );
  void setScale(const RvVec3 &scale);

  // intersect
 int intersect(const RvRay &r,RvVec &pos,RvVec &norm,float &d,int backf_cull=1)
 { 
	 float t1, t2;
	 return int(boundingBox.intersect(r, t1, t2)); 
 }

};

#endif
