#ifndef RV_QUAT_H
#define RV_QUAT_H

struct RvQuat : public RvVec4
{
	
	RvQuat() { *this = RvQuat::identity(); }

	RvQuat(const RvVec3 &vec, float s) 
	{ c[0]=vec[0]; c[1]=vec[1]; c[2]=vec[2]; c[3]=s; }
	RvQuat(const RvVec3d &vec, double s) 
	{ c[0]=vec[0]; c[1]=vec[1]; c[2]=vec[2]; c[3]=s; }

	RvQuat(float x,float y,float z, float s)
	{ c[0]=x; c[1]=y; c[2]=z; c[3]=s; }

	RvQuat(const RvQuat &q) {*this = q; }

	RvQuat(const double *xyz)
	{  for(int i=0;i<4;i++) c[i]=xyz[i]; }
	RvQuat(const float *xyz)
	{ for(int i=0;i<4;i++) c[i]=xyz[i]; }

	RvQuat(const RvVector<double,4> &v) 
	{ c[0]=v[0]; c[1]=v[1]; c[2]=v[2]; c[3]=v[3];}
	RvQuat(const RvVector<float,4> &v) 
	{ c[0]=v[0]; c[1]=v[1]; c[2]=v[2]; c[3]=v[3];}
	
	RvVec getAxis()  const { 
		RvVec v;
		v[0]=c[0]; v[1]=c[1]; v[2]=c[2];
		v.normalize();
		return v; 
	}
	float getAngle()  const { return 2*acos(c[3]);}
	void setAxisAngle(const RvVec &axis, float angle)
	{ setAxis(axis); setAngle(angle); normalize(); }

	RvVec getVector() const
	{
		return RvVec(c[0],c[1],c[2]);
	}


	void conj()
	{ c[0]=-c[0]; c[1]=-c[1]; c[2]=-c[2];}
	RvQuat getConj() const
	{ return RvQuat(-c[0],-c[1],-c[2],c[3]); }

	
	void toBallPoints(RvVec &from, RvVec &to) const
	{
		RvQuat q = *this;
		q.normalize();
		double s = sqrt(q.c[0]*q.c[0]+q.c[1]*q.c[1]);
		if (s==0.0)
			from = RvVec(0,1,0);
		else
			from = RvVec(-q.c[1]/s,q.c[0]/s,0.0);

		to[0] = q.c[3]*from[0]  - q.c[2]*from[1];
		to[1] = q.c[3]*from[1]  + q.c[2]*from[0];
		to[2] = q.c[0]*from[1]  - q.c[1]*from[0];
		
		if (q.c[3]<0.0) from = RvVec(-from[0],-from[1],0.0);
	}

  void toAxisAngle( float *angle, RvVec *axis ) {
    if ( c[3] > 1.0  ) c[3] = 1.0;
    if ( c[3] < -1.0 ) c[3] = -1.0;
    float omega = acosf( c[3] );
    float sinus = sinf( omega );

    if ( fabsf( sinus ) < 0.001 ) {
      *angle = 1;
      *axis  = 0.0;
    } else {
      *angle = ( omega * 360.0 ) / RV_M_PI;
      (*axis)[ 0 ]  = c[ 0 ] / sinus;
      (*axis)[ 1 ]  = c[ 1 ] / sinus;
      (*axis)[ 2 ]  = c[ 2 ] / sinus;

    }
  }

	void applyQuat(const RvQuat &q)
	{
		RvVec v1,v2;
		v1[0]=c[0]; v1[1]=c[1]; v1[2]=c[2];
		v2[0]=q[0]; v2[1]=q[1]; v2[2]=q[2];
		RvVec V = c[3]*v2 + q[3]*v1 + v2.cross(v1);
		c[3]=c[3]*q[3] - v1.dot(v2);
		c[0]=V[0]; c[1]=V[1]; c[2]=V[2];
	}


	void applyTo(RvVec &V) const
	{
		float m[4*4];
		getRotationMatrix(m);
		
		RvVec res = 0.0;
		for(int i=0;i<3;i++)
			for(int j=0;j<3;j++) res[i] += m[j*4+i]*V[j];
		V= res;
	}

	void getRotationMatrix(float *m) const
	{
		float t = 2.0 / (getVector().norm2() + c[3]*c[3]);

		float xs = c[0]*t; float ys = c[1]*t; float zs = c[2]*t;
		float wx = c[3]*xs; float wy = c[3]*ys; float wz = c[3]*zs;
		float xx = c[0]*xs; float xy = c[0]*ys; float xz = c[0]*zs;
		float yy = c[1]*ys; float yz = c[1]*zs; float zz = c[2]*zs;

	m[0*4+0] = 1.0-(yy+zz); m[0*4+1]=xy+wz; m[0*4+2] = xz-wy; m[0*4+3]=0.0;
	m[1*4+0] = xy-wz; m[1*4+1] = 1.0-(xx+zz); m[1*4+2]=yz+wx; m[1*4+3]=0.0;
	m[2*4+0] = xz+wy; m[2*4+1] = yz-wx; m[2*4+2]=1.0-(xx+yy); m[2*4+3]=0.0;
	m[3*4+0] = 0.0; m[3*4+1] = 0.0; m[3*4+2]= 0.0; m[3*4+3]= 1.0;
/*
	float XX =  c[0] +  c[0];
	float YY =  c[1] +  c[1];
	float ZZ =  c[2] +  c[2];

	float WXX = c[3] * XX;
	float XXX = c[0] * XX;

	float WYY = c[3] * YY;
	float XYY = c[0] * YY;
	float YYY = c[1] * YY;

	float WZZ = c[3] * ZZ;
	float XZZ = c[0] * ZZ;
	float YZZ = c[1] * ZZ;
	float ZZZ = c[2] * ZZ;

	m[0] = (float) 1.0 - (YYY + ZZZ);
	m[1] = (float) XYY - WZZ;
	m[2] = (float) XZZ + WYY;
	m[3] = 0;

	m[4] = (float) XYY + WZZ;
	m[5] = (float) 1.0 - (XXX + ZZZ);
	m[6] = (float) YZZ - WXX;
	m[7] = 0;

	m[8] = (float) XZZ - WYY;
	m[9] = (float) YZZ + WXX;
	m[10] = (float) 1.0 - (XXX + YYY);
	m[11] = 0;

	m[12] = 0;
	m[13] = 0;
	m[14] = 0;
	m[15] = 1;
*/
  }


	static RvQuat compose(const RvQuat &q1,const RvQuat &q2)
	{
		RvQuat Q= q2;
		Q.applyQuat(q1);
		return Q;
	}

	static RvVec apply(const RvVec &v,const RvQuat &q)
	{
		RvVec z = v;
		q.applyTo(z); return z;
	}

	static RvQuat fromMatrixGL(float *mat)
	{
		RvQuat q;
    		q[3] = sqrt(1.0 + mat[0] + mat[5] + mat[10]) / 2.0;
		double w4 = (4.0 * q[3]);
		q[0] = (mat[6] - mat[9]) / w4 ;
		q[1] = (mat[8] - mat[2]) / w4 ;
		q[2] = (mat[1] - mat[4]) / w4 ;
		q.normalize();
		return q;
	}

	static RvQuat fromBallPoints(const RvVec &from, const RvVec &to)
	{
		RvVec p1,p2; p1=from; p2=to;
		p1.normalize(); p2.normalize();
		RvVec P = p1.cross(p2);
		//RvQuat q(P,p1.dot(p2));
		RvQuat q;
		q.setAxisAngle(P,acos(p1.dot(p2)));
		q.normalize();
		return q;
	}
	static RvQuat fromAxisAngle(const RvVec &v,float a)
	{
		RvQuat q; q.setAxisAngle(v,a);
		return q;
	}

  void fromEuler(float roll, float pitch, float yaw) {
	  float cr,cp,cy,sr,sp,sy,cpcy,spsy;

	  cr = (float) cosf( roll  / 2.0f);
	  cp = (float) cosf( pitch / 2.0f);
	  cy = (float) cosf( yaw   / 2.0f);
	  sr = (float) sinf( roll  / 2.0f);
	  sp = (float) sinf( pitch / 2.0f);
	  sy = (float) sinf( yaw   / 2.0f);

	  cpcy = cp * cy;
	  spsy = sp * sy;

	  c[3] = cr * cpcy + sr * spsy;
	  c[0] = sr * cpcy - cr * spsy;
	  c[1] = cr * sp * cy + sr * cp * sy;
	  c[2] = cr * cp * sy - sr * sp * cy;
  }

	static RvQuat identity()
		{ RvQuat q(0,0,0,1); return q; }

	static RvQuat slerp(const RvQuat &start,const RvQuat &end, float c)
	{
    double omega, sinom, scale0, scale1;
		RvQuat end1;
		double cosom = start.getVector().dot(end.getVector())
				+ start.c[3]*end.c[3];

		if (cosom<0.0)
		{
			cosom = -cosom;
			end1 = -end;
		}
		else
		{
			end1 = end;
		}

		if((1.0-cosom)>1e-5)
		{
			omega = acos(cosom);
			sinom = sin(omega);
			scale0 = sin((1.0-c) * omega)/ sinom;
			scale1 = sin(c*omega)/sinom;
		}
		else
		{
			scale0 = 1.0-c;
			scale1 = c;
		}

		return (scale0 * start + scale1*end1);
	}


protected:
	void setAngle(float _a) {
		 RvVec ax = getAxis(); 
		 c[3] = cos(_a/2.0);
		 RvVec v = ax * sin(_a/2.0);
		 c[0]=v[0]; c[1]=v[1]; c[2]=v[2];
        }
	void setAxis(const RvVec &v)
	{
		RvVec V=v; V.normalize();
		c[0]=V[0]; c[1]=V[1]; c[2]=V[2];
	}
};

#endif
