#ifndef RV_SPACE_H
#define RV_SPACE_H

#include "RvVec.h"
#include "RvGeom.h"
#include "RvModel.h"

#include <assert.h>

class RvModel;

//////////////////////////////////////////////////////////////////

struct RvPList 
{
	RvPList() {primitive=NULL; next=NULL; nb=0;}

	void insert(RvModel *prim)
	{
		if (nb==0)
		{
			assert(primitive==NULL);
			assert(next==NULL);
			primitive = prim;
		}
		else
		{
			RvPList *p = new RvPList;
			p->next = next;
			next = p;
			p->primitive = prim;
		}
		nb++;
	}

        RvModel *primitive;
        RvPList *next;
	int nb;
};


//////////////////////////////////////////////////////////////////

struct RvSpace
{
	virtual RvModel * findClosestPrimitive(RvModel *,float radius,int type) =0;
	virtual RvModel * intersect(const RvRay &r,RvVec &P,RvVec &N,float &d) = 0 ;

 	virtual void draw(); 

	virtual void load(char *fname);

	virtual void scale(const RvVec &s);

	virtual void buildSpace() {}

	virtual RvPList & getPrimitiveList() { return list;}

	RvBox bbox;


protected:
	RvSpace()  {}

	RvPList list;
};

//////////////////////////////////////////////////////////////////

struct RvPrimitiveList : public RvSpace
{
	RvPrimitiveList() {}
	
	virtual RvModel * findClosestPrimitive(RvModel *,float radius,int type);
	virtual RvModel * intersect(const RvRay &r,RvVec &P,RvVec &N,float &d);
};

//////////////////////////////////////////////////////////////////

struct RvCell
{
	RvCell() { empty=1;}
	RvPList *list;
	int empty;
	void *userData;
};

struct RvGridSpace : public RvSpace
{
	RvGridSpace() { nbx = nby= nbz = 5; cells =NULL;}

	virtual RvModel * findClosestPrimitive(RvModel *,float radius,int type);
	virtual RvModel * intersect(const RvRay &r,RvVec &P,RvVec &N,float &d);

	virtual void setSubdivision(int x,int y,int z) { nbx = x; nby=y,nbz=z;}
	virtual void buildSpace();

	virtual void drawCells();
	virtual void clearCells() { for(int i=0;i<nbx*nby*nbz;i++) cells[i].empty=1;}

protected:
	int nbx,nby,nbz;

	inline RvVec getCellIndex(const RvVec &pos)
	{
		RvVec p;
		p= (pos-bbox.min)/cellSize;
		p[0] = RV_MIN(nbx-1,floorf(p[0]));
		p[1] = RV_MIN(nby-1,floorf(p[1]));
		p[2] = RV_MIN(nbz-1,floorf(p[2]));
		
		return p;
	}

	inline RvVec getMinCellPos(const RvVec &cellindex)
	{
		return cellSize*cellindex+bbox.min;
	}

	RvCell *cells;
	RvVec cellSize;
};

#endif
