#include "RvSys.h"
#include <iostream>
#include <assert.h>
#include <stdlib.h>

#ifdef _WIN32
	#include <afxmt.h>
	#include <winbase.h>
	#include <mmsystem.h>
	#include <winsock2.h>
	#include <wtypes.h>
#endif

static aluint Rvsys_inited=0;


#if defined (irix)
	usptr_t *RV_ARENA(NULL);
#endif

#ifdef _WIN32

   static WSADATA      wsd;
   static LONGLONG RV_GLOBRV_FREQ;
   static LONGLONG RV_GLOBRV_COUNT;

#endif

static double RV_GLOBRV_TIME;

aluint RvInitSys()
{

	if (Rvsys_inited) return 1;

	std::cerr << "AL system init" << std::endl;

#if defined (irix) 
        usconfig(CONF_INITUSERS,5000);
        usconfig(CONF_INITSIZE,1000000);
        usconfig(CONF_LOCKTYPE,US_NODEBUG);

        struct rlimit L;
        L.rlim_cur= 1000;
        L.rlim_max= 1000;

        setrlimit(RLIMIT_PTHREAD, &L);
        setrlimit(RLIMIT_NOFILE, &L);

	RV_ARENA = usinit(tmpnam(0));
	assert(RV_ARENA);
#endif


#ifdef irix
	struct timeval _time;
	gettimeofday(&_time);
#endif

#ifdef linux
	struct timeval _time;
	struct timezone t_zone;
	gettimeofday(&_time,&t_zone);
#endif

#if defined (irix) || defined (linux)
	RV_GLOBRV_TIME = _time.tv_sec + _time.tv_usec*1e-6;	

	//srand48(time(NULL));

#endif

#ifdef _WIN32

    // Initialize the Winsock2 DLL
    //
//   assert(!WSAStartup(MAKEWORD(2, 2), &wsd));

   //srand(unsigned(time(NULL)));

   // RV_GLOBRV_TIME = GetTickCount()/1000.0;
 
   LARGE_INTEGER A;

   QueryPerformanceFrequency(&A);
   RV_GLOBRV_FREQ = A.QuadPart;
   if (RV_GLOBRV_FREQ)
   {
	std::cerr << "using high rez timer @ " << (long)RV_GLOBRV_FREQ << " Hz" << std::endl;
   	QueryPerformanceCounter(&A);
	RV_GLOBRV_COUNT = A.QuadPart;
	RV_GLOBRV_TIME = 0.0;
   }
   else
   {
   	timeBeginPeriod(1);
	std::cerr << "using low rez timer @ 1000 Hz" << std::endl;
   	RV_GLOBRV_TIME = timeGetTime()/1000.0;
  }

#endif

	Rvsys_inited = 1;

	return 1;
}

double RvGetSysTime()
{
#ifdef _WIN32
	//return GetTickCount()/1000.0 - RV_GLOBRV_TIME;

	if ((long)RV_GLOBRV_FREQ)
	{
		LARGE_INTEGER A;
   		QueryPerformanceCounter(&A);
		return(1.0*(A.QuadPart-RV_GLOBRV_COUNT)/RV_GLOBRV_FREQ);
	}
	else
		return timeGetTime()/1000.0 - RV_GLOBRV_TIME;
#endif

#if defined (irix) || defined (linux)
	struct timeval _time;

#ifdef irix
	gettimeofday(&_time);
#endif

#ifdef linux
	struct timezone t_zone;
	gettimeofday(&_time,&t_zone);
#endif

	return (_time.tv_sec + _time.tv_usec*1e-6) - RV_GLOBRV_TIME;
#endif
}

void RvSleep(aluint t)
{
#ifdef irix
	sginap((aluint)(1.0*t*CLK_TCK/1000.0));
#endif
#ifdef linux
	usleep((aluint)(t*1000.0)); // usleep is in microsec.
#endif

#ifdef _WIN32

	Sleep(t);
#endif

}






