#ifndef RV_TRACKBALL_H
#define RV_TRACKBALL_H

#include "RvVec.h"
#include "RvQuat.h"

#include <iostream>

struct RvTrackBall
{
	RvTrackBall()
	{
		center = RvVec(0,0,0);
		radius = 1.0;
		startx = x = starty = y = 0;
		drag = 0;
		w = h= 0.0;
	}

	void setPosition(RvVec &_center,float _radius=-1)
	{
		center = _center; 
		if (radius>0.0) setRadius(_radius);
	}
	void setRadius(float _radius) { radius = _radius;}

	const RvVec &getCenter() const { return center;}
	float getRadius() const { return radius;}

	const RvQuat &getQuaternion() const {return quat;}

	void beginDrag(int mousex,int mousey,int width,int height) 
	{ 
		drag = 1; startx = 2.0*mousex/width -1.0;
		starty = 2.0*mousey/height - 1.0;
		starty *=-1;
		w = width; h = height;
		from = mouseOnSphere(startx,starty);
	}
	void endDrag() { drag = 0; qstart = quat;}
	void update(int mousex,int mousey)
	{
		if (!drag) return;

		x = 2.0*mousex/w-1.0; y = 2.0*mousey/h-1.0;
		y *=-1;

		to = mouseOnSphere(x,y);

		RvQuat rot = RvQuat::fromBallPoints(from,to);	
		quat = RvQuat::compose(rot,qstart);
	}

	void getMatrix(float *m)
	{ quat.getRotationMatrix(m); }

	void reset(RvQuat q = RvQuat::identity() ) {qstart = q;}

protected:

	RvVec mouseOnSphere(float mousex,float mousey)
	// mousex , mousey in [-1,1]
	{
		float X = (mousex - center[0])/2.0;	
		float Y = (mousey - center[1])/2.0;	

		float d = X*X+Y*Y;
	
		if (d < radius * RV_M_SQRT2_2)
			return RvVec(X,Y,sqrt(radius*radius - d*d));
		else
		{
			double t = radius/RV_M_SQRT2;
			return RvVec(X,Y,t*t/d);
		}
	}


	RvVec center;
	float radius;

	int drag;
	int w,h;
	float x,y;
	float startx,starty;
	RvVec from, to;

	RvQuat quat;
	RvQuat qstart;
};



#endif
