#ifndef RV_VEC_H
#define RV_VEC_H

#include <math.h>
#include <iostream>
//#include "RvDefs.h"


template <class _Type_,int SIZE>
struct RvVector
{
	RvVector() {}
	RvVector(const double *xyz)
	{  for(int i=0;i<SIZE;i++) c[i]=xyz[i]; }
	RvVector(const float *xyz)
	{ for(int i=0;i<SIZE;i++) c[i]=xyz[i]; }
	RvVector(const _Type_ &x) { for(int i=0;i<SIZE;i++) c[i]=x;}
	RvVector(const _Type_ &x,const _Type_ &y)
	{ c[0]=x; c[1]=y; }
	RvVector(const _Type_ &x,const _Type_ &y,const _Type_ &z)
	{ c[0]=x; c[1]=y; c[2] = z;}
	RvVector(const _Type_ &x,const _Type_ &y,const _Type_ &z, const _Type_&w)
	{ c[0]=x; c[1]=y; c[2] = z; c[3]=w;}


	~RvVector(){}

    	 _Type_ & operator[]( int i) {return c[i];}
    	 const  _Type_ &operator[]( int i) const {return c[i];}


	 RvVector &operator = (const RvVector &src) {  for(int i=0;i<SIZE;i++) c[i]=src[i]; return *this;}
	 RvVector &operator = (const float *src) {  for(int i=0;i<SIZE;i++) c[i]=src[i]; return *this;}
	 RvVector &operator = (const  double *src) {  for(int i=0;i<SIZE;i++) c[i]=src[i]; return *this;}
	 RvVector &operator=  (const _Type_ &a) { for(int i=0;i<SIZE;i++) c[i]=a; return *this;}

	 RvVector& operator/=( const _Type_ &a) { for(int i=0;i<SIZE;i++) c[i]/=a; return *this;}
	 RvVector& operator+=( const  _Type_ &a) { for(int i=0;i<SIZE;i++) c[i]+=a; return *this;}
	 RvVector& operator-=( const  _Type_ &a) { for(int i=0;i<SIZE;i++) c[i]-=a; return *this;}
	 RvVector& operator*=( const  _Type_ &a) { for(int i=0;i<SIZE;i++) c[i]*=a; return *this;}

	 RvVector& operator/=( const RvVector& a) { for(int i=0;i<SIZE;i++) c[i]/=a[i]; return *this;}
	 RvVector& operator+=( const RvVector& a) { for(int i=0;i<SIZE;i++) c[i]+=a[i]; return *this;}
	 RvVector& operator-=( const RvVector& a) { for(int i=0;i<SIZE;i++) c[i]-=a[i]; return *this;}
	 RvVector& operator*=( const RvVector& a) { for(int i=0;i<SIZE;i++) c[i]*=a[i]; return *this;}

	 bool operator <= (const RvVector &src) {  bool tmp = true; for(int i=0;i<SIZE;i++) tmp &= (c[i]<=src[i]); return tmp;}
	 bool operator >= (const RvVector &src) {  bool tmp = true; for(int i=0;i<SIZE;i++) tmp &= (c[i]>=src[i]); return tmp;}
	 bool operator <  (const RvVector &src) {  bool tmp = true; for(int i=0;i<SIZE;i++) tmp &= (c[i]<src[i]); return tmp;}
	 bool operator >  (const RvVector &src) {  bool tmp = true; for(int i=0;i<SIZE;i++) tmp &= (c[i]>src[i]); return tmp;}

	 RvVector &Cartesian2Polar()
        {
                 _Type_ x = c[0];
                 _Type_ y = c[1];
                 _Type_ z = c[2];
                c[0]= sqrt(x*x+y*y+z*z);        //r
		if (c[0]==0.0) {*this=0.0; return *this;}
		if (x!=0.0)
                	c[1] =atan2(z,x);       // theta (X/Z plane) in [-pi,pi]
		else
			c[1] = 0;
                c[2] =RV_M_PI_2-acos(y/c[0]);   //phi (to Y axis) in [-pi/2,pi/2]
                return *this;
        }
         RvVector &Polar2Cartesian()
        {
                 _Type_ r = c[0];
                 _Type_ theta = c[1];
                 _Type_ phi = RV_M_PI_2-c[2];
                c[2] = r*sin(theta)*sin(phi);
                c[1] = r*cos(phi);
                c[0] = r*cos(theta)*sin(phi);
                return *this;
        }

	 int operator==(const RvVector&a)
	{
		int iseq =1;
		for(int i=0;i<SIZE;i++)
			iseq = iseq && ( c[i]==a[i] );
		return iseq;
	}

	 friend RvVector operator -(const RvVector &a)
		{ RvVector b = a; b*=-1; return b;}

	 friend RvVector operator/(const RvVector& s, const  _Type_ &a)
		{RvVector S; for(int i=0;i<SIZE;i++) S[i]=s[i]/a; return S;}
	 friend RvVector operator-(const RvVector& s, const  _Type_ &a)
		{RvVector S; for(int i=0;i<SIZE;i++) S[i]=s[i]-a; return S;}
	 friend RvVector operator+(const RvVector& s, const  _Type_ &a)
		{RvVector S; for(int i=0;i<SIZE;i++) S[i]=s[i]+a; return S;}
	 friend RvVector operator*(const RvVector& s, const  _Type_ &a)
		{RvVector S; for(int i=0;i<SIZE;i++) S[i]=s[i]*a; return S;}

	 friend RvVector operator/(const RvVector& s, const RvVector& a)
		{RvVector S; for(int i=0;i<SIZE;i++) S[i]=s[i]/a[i]; return S;}
	 friend RvVector operator-(const RvVector& s, const RvVector & a)
		{RvVector S; for(int i=0;i<SIZE;i++) S[i]=s[i]-a[i]; return S;}
	 friend RvVector operator+(const RvVector& s, const RvVector & a)
		{RvVector S; for(int i=0;i<SIZE;i++) S[i]=s[i]+a[i]; return S;}
	 friend RvVector operator*(const RvVector& s, const RvVector& a)
		{RvVector S; for(int i=0;i<SIZE;i++) S[i]=s[i]*a[i]; return S;}
	 friend std::ostream& operator<<(std::ostream& os , const RvVector& v)
		{for(int i=0;i<SIZE;i++) { os << v[i] << " " ; } return os;}


   	friend std::ifstream& operator >> ( std::ifstream& is, RvVector& v ) {
    		for( int i = 0; i < SIZE; i++ )
      		is >> (_Type_)v[i];
    		return is;
   	}


	 RvVector cross(const RvVector &v2)
	{
		 return RvVector
				(c[1]*v2[2] - v2[1]*c[2],
			     v2[0]*c[2] - c[0]*v2[2],
			     c[0]*v2[1] - v2[0]*c[1]
		);
	}

	 _Type_ dot(const RvVector &ref) const {
		_Type_ d =0.0;
		for(int i=0;i<SIZE;i++)  d+= c[i]*ref[i];
		return d;
	}

	 _Type_ length() const {
		_Type_ d =0.0;
		for(int i=0;i<SIZE;i++)  d+= c[i]*c[i];
		return sqrt(d);
	}

	 _Type_ norm2() const {
		_Type_ d =0.0;
		for(int i=0;i<SIZE;i++)  d+= c[i]*c[i];
		return d;
	}

	 _Type_ normalize()
		{ _Type_ a = length() ; if (a!=0.0) (*this)/=a; return a;}

	 _Type_ mini() const {
		_Type_ m = 1e10;
		for(int i=0;i<SIZE;i++)  if (c[i]<m) m = c[i];
		return m;
	}
	 _Type_ maxi() const {
		_Type_ m = -1e10;
		for(int i=0;i<SIZE;i++)  if (c[i]>m) m = c[i];
		return m;
	}

	 _Type_ avg() const {
		_Type_ m = 0;
		for(int i=0;i<SIZE;i++)  m+=c[i];
		return m/SIZE;
	}

protected:
	 _Type_ c[SIZE];
};

typedef RvVector<float,3> RvVec;
typedef RvVector<float,3> RvVec3;
typedef RvVector<double,3> RvVec3d;
typedef RvVector<float,4> RvVec4;
typedef RvVector<double,4> RvVec4d;
typedef RvVector<float,2> RvVec2;



#endif
