#include "gfxio.h"
#include "tiffio.h"
//#include "tiffiop.h"
#include <iostream>

int saveTiffRGB(char *name, int x,int y,char *buffer, int flags)
{
	short config = PLANARCONFIG_CONTIG;
	uint16 compression = COMPRESSION_LZW;
	uint16 predictor = 0;
	uint32 rowsperstrip = (uint32) -1;
	int jpegcolormode = JPEGCOLORMODE_RGB;
	int quality = 75;		/* JPEG quality */
	uint16 photometric;

	TIFF *out = TIFFOpen(name, "w");
	if (out == NULL) return (-2);

	TIFFSetField(out, TIFFTAG_IMAGEWIDTH, x);
	TIFFSetField(out, TIFFTAG_IMAGELENGTH, y);
	TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, 8);
#ifndef linux
	TIFFSetField(out, TIFFTAG_COMPRESSION, compression);
#endif
	
	photometric = PHOTOMETRIC_RGB;

	switch (compression) {
	case COMPRESSION_JPEG:
		if (photometric == PHOTOMETRIC_RGB && jpegcolormode == JPEGCOLORMODE_RGB)
			photometric = PHOTOMETRIC_YCBCR;
		TIFFSetField(out, TIFFTAG_JPEGQUALITY, quality);
		TIFFSetField(out, TIFFTAG_JPEGCOLORMODE, jpegcolormode);
		break;
	case COMPRESSION_LZW:
	case COMPRESSION_DEFLATE:
		if (predictor != 0)
			TIFFSetField(out, TIFFTAG_PREDICTOR, predictor);
		break;
	}

	TIFFSetField(out, TIFFTAG_PHOTOMETRIC, photometric);

	TIFFSetField(out, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
	TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 3);
	TIFFSetField(out, TIFFTAG_PLANARCONFIG, config);

	TIFFSetField(out, TIFFTAG_ROWSPERSTRIP, TIFFDefaultStripSize(out, rowsperstrip));

	tdata_t buf = _TIFFmalloc(TIFFScanlineSize(out));


	for (int Y = y-1; Y >= 0; Y--) {
		uint8* pp = (uint8*) buf;
		unsigned char *B; 	
		if (flags & GFXIO_FLIP_VERTICAL)
			B = (unsigned char *) &buffer[3*x*Y];
		else
			B = (unsigned char *) &buffer[3*x*(y-Y-1)];

		int incr = 3;
		if (flags & GFXIO_FLIP_HORIZONTAL)
			B += 3*(x-1);

		for (int X = 0; X < x; X++) {
			pp[0] = B[0];
			pp[1] = B[1];
			pp[2] = B[2];
			pp += 3;
			B +=incr;
		
		}
		if (TIFFWriteScanline(out, buf, y-Y-1, 0) < 0)
			goto bad;
	}
	_TIFFfree(buf);
	(void) TIFFClose(out);
	return (0);
bad:
	_TIFFfree(buf);
	return (-1);
}

int loadTiff(char *name, char *buffer,int *x,int *y,int *pixeldepth)
{
	TIFF *out = TIFFOpen(name, "r");
	if (out == NULL) return (-2);

	uint16 pp;

	TIFFGetField(out, TIFFTAG_SAMPLESPERPIXEL, &pp);
	TIFFGetField(out, TIFFTAG_IMAGEWIDTH, x);
	TIFFGetField(out, TIFFTAG_IMAGELENGTH, y);

	*pixeldepth = pp;

	std::cerr << "w = " << *x << " h = " << *y << std::endl;
	std::cerr << "depth = " << *pixeldepth << std::endl;
	
	if (!buffer) 
	{
		(void) TIFFClose(out);
		return -1;
	}

	tdata_t buf = _TIFFmalloc(TIFFScanlineSize(out));

	unsigned char *B = (unsigned char *)buffer;

	for (int Y = *y-1; Y >= 0; Y--) {
		uint8* pp = (uint8*) buf;
		TIFFReadScanline(out, buf, *y-Y-1, 0);
		for (int X = 0; X < *x; X++) {
			for(int zz = 0;zz<*pixeldepth;zz++)
				B[zz] = pp[zz];
			B += *pixeldepth;
			pp+=*pixeldepth;
		}
	}

	(void) TIFFClose(out);
	return (0);
}
