#ifndef _FF_H
#define _FF_H

/*!
  \file ff.h
  \brief 3d file format declaratoins.

  \author Eric Paquette
  \date Wed Jun  2 17:40:45 1999

  This file contains declarations related to the 3d scene / model file
  format.
*/


/*
******************************************************************************
**
** Structures
**
******************************************************************************
*/


/*! Struct used to store vertices positions when reading the model
  description file. */ 
//typedef struct t_ffVertex
class ffVertex
{
public:
  double xyz[3];
  int index;
  int nindex, tindex;
  ffVertex() { nindex = tindex = 0; index =0;} 
} ; //ffVertex;

/*! Struct used to store vertices positions when reading the model
  description file. */ 
typedef struct t_ffNormal
{
  double xyz[3];
  int index;
} ffNormal;

/*! Struct used to store texture coordinates when reading the model
  description file. */ 
typedef struct t_TextureCoord
{
  double uvw[3];
  int index;
} TextureCoord;

/*! Struct used to store vertices index and texture index when reading the
  model description file. */ 
typedef struct t_Vertices
{
  int index;
  int maxIndex;
  int* xyzi; /* xyz index */
  int* uvi;  /* uv index */
  int* ni;
  int usedUV;
  int usedN;
} Vertices;

/*!Struct used when converting files (compressing vertices) to store vertices
  coordinates. */
typedef struct t_VertexSave
{
  int bufSize;
  int i;
  char*  vertex;
  char*  space[ 2 ];
  char*  coordText[ 3 ];
  double coord[ 3 ];
} VertexSave;

typedef struct t_CompressionVertices
{
  double x;
  double y;
  double z;
  int link;
} CompressionVertices;

typedef struct t_Features
{
  long vertices;
  long normals;
  long textures;
  long groups;
  long smoothGroups;
  long materials;
  long maps;
  long shadowObjects;
  long mapLibs;
  long mtlLibs;
  long lines;
  long faces;
  long bezierPatches;
} Features;

enum TextureType { kRandom };


/*
******************************************************************************
**
** Forward declarations
**
******************************************************************************
*/


class BaseShader;
class Face;
class Model;
template<class T> class Point3d;


// TBD:Eric Paquette 2000/10/02: I have to fix this somehow, this should be
// defined some place where I wouldn't need to include it in model.h. May be
// adding a pointer level is the best solution.
struct FaceTriIndexes
{
  int indexes[3];
};


/*
******************************************************************************
**
** Externs declaration
**
******************************************************************************
*/
extern ffVertex gVertex;
extern ffNormal gNormal;
extern Vertices gVertices;
extern TextureCoord gTextureCoord;
extern int gTriangleVertexIndex[3];
extern Features gFeatures;

extern VertexSave gSave;
extern char* gSaveString;
extern int gSaveStringSize;

extern int gNumCompressionVertices;
extern int gMaxCompressionVertices;
extern CompressionVertices* gCompressionVertices;
extern int gNumCompressedVertices;
extern int gMaxCompressedVertices;
extern CompressionVertices* gCompressedVertices;

extern int gCurrentShader;

extern float gTextureR;
extern float gTextureG;
extern float gTextureB;
extern bool  gHasTextureCoef;
extern bool  gHasTextureMap;

extern bool  gHasNiCoeff;
extern bool  gHasIllCoeff;
extern float gIllCoeff;
extern float gNiCoeff;

extern float gAmbientR;
extern float gAmbientG;
extern float gAmbientB;
extern bool  gHasAmbient;

extern float gDiffuseR;
extern float gDiffuseG;
extern float gDiffuseB;
extern bool  gHasDiffuse;

extern float gSpecularR;
extern float gSpecularG;
extern float gSpecularB;
extern bool  gHasSpecular;

extern float gSpecularN;
extern bool  gHasSpecularN;


#define YY_NEVER_INTERACTIVE 1

/*
******************************************************************************
**
** Functions definition
**
******************************************************************************
*/


void ResetVertexInfo();
void ResetNormalInfo();
void ResetTextureCoordinateInfo();
void ResetVerticesInfo();
void ResetSavedInfo();
void ResetMaterialInfo();

int GetShader( const char* szMaterialName );
void CloseFaceGroup();

void AddVertex( ffVertex v );
void AddNormal( ffNormal n );
void AddTextureCoordinate2d( TextureCoord* t );

void AddFace( int* vertices, int numVertices );
void AddFace( int* vertices, int* normals, int numVertices );
void AddFace( int* vertices, int* uvi, int* normals, int numVertices );

void AddColor( double r, double g, double b );
void AddDisplacementMap( /*enum TextureType*/ );

void AddShader( char* szMaterialName );

int readModelFile( const char* szFileName, bool report = true );

bool IsDefaultMaterialFile( const char* szName );
void LoadMaterialFile( const char* filename );
void LoadTextureFile( const char* filename );

int convertModelFile( const char* szFileName, const char* szOutput );

void ShutdownFileParser();
bool yy1Init( const char* szFileName );
void yy1End();

bool yymtlInit( const char* szFileName );
void yymtlEnd();

/* End of file */
/****************************************************************************/
#endif /* _FF_H */
