%{
#include <stdlib.h>
#include <stdio.h>
#if (defined(__sgi) && (_COMPILER_VERSION >= 721))
#pragma set woff 1110,1174,1516,1167,3322
#endif
#define PRINT(a,b)
//#define PRINT(a,b) printf( a, b )
%}

%option yylineno
%option noyywrap
%s cFACE
%x cGROUP cSMOOTH_GROUP cMTLLIB

WHITE           [\t\n\ ]
NONWHITE        [^\t\n\ ]

NEWLINE         \n

SPACE           [\t\ ]+

SIGN            [+-]
DIGIT           [0-9]
INTEGER         {SIGN}?{DIGIT}+
FLOAT           {SIGN}?{DIGIT}*"."{DIGIT}*
FLOATEXP        {FLOAT}[eE]{SIGN}?{INTEGER}

SEPARATOR       [\/;:]

VERTEX          v
TEXTURE_COORD   vt
VERTEX_NORMAL   vn
GROUP           g
SMOOTH_GROUP    s
OBJECT          o
CINTERP         c_interp

MATERIAL        material
DISPLACEMENTMAP displacement-map
RANDOM          random

USE_MATERIAL    usemtl
USE_MAP         usemap
SHADOW_OBJECT   shadow_obj
MATERIAL_LIB    mtllib
MAP_LIB         maplib
RES             res

TRIANGLE        [tT]
QUADRILATERAL   [qQ]
FACE            fo?
LINE            l
BEZIER_PATCH    bzp

QUOTED_STRING   \"[^"]*(\"\"[^"]*)*\"
ALPHA           [a-zA-Z]
ALPHANUM        ({ALPHA}|[0-9])
ALPHAEXT        {ALPHANUM}|"-"|_|"."|"/"
ID              {ALPHA}({ALPHAEXT})*
SUPERID         {ALPHAEXT}({ALPHAEXT})*

%%

{VERTEX}          { return( VERTEX ); }
{TEXTURE_COORD}   { return( TEXTURE_COORD ); }
{VERTEX_NORMAL}   { return( VERTEX_NORMAL ); }

{QUADRILATERAL}   {
  PRINT( "quadrilateral face %s\n", yy1text );
  BEGIN( cFACE );
  PRINT( "state %d\n", yy_start );
  return( QUADRILATERAL );
}
{TRIANGLE}        {
  PRINT( "triangle face %s\n", yy1text );
  BEGIN( cFACE );
  return( TRIANGLE );
}
{FACE}            {
  PRINT( "face %s\n", yy1text );
  BEGIN( cFACE );
  PRINT( "state %d\n", yy_start );
  return( FACE );
}
{LINE}            { return( LINE ); }
{BEZIER_PATCH}    { return( BEZIER_PATCH ); }
{OBJECT}          { return( OBJECT ); }

{MATERIAL}        { return( MATERIAL ); }
{DISPLACEMENTMAP} { return( DISPLACEMENTMAP ); }
{RANDOM}          { return( RANDOM ); }
{USE_MAP}         { return( USE_MAP ); }
{USE_MATERIAL}    { return( USE_MATERIAL ); }
{SHADOW_OBJECT}   { return( SHADOW_OBJECT ); }
{MATERIAL_LIB}    {
  PRINT( "mtllib %s\n", yy1text );
  BEGIN( cMTLLIB );
  PRINT( "state %d\n", yy_start );
  return( MATERIAL_LIB );
}
{MAP_LIB}         { return( MAP_LIB ); }
{CINTERP}         { return( CINTERP ); }
{RES}             { return( RES ); }

{GROUP}           {
  PRINT( "group %s\n", yy1text );
  BEGIN( cGROUP );
  PRINT( "state %d\n", yy_start );
  return( GROUP );
}

{SMOOTH_GROUP}    {
  PRINT( "smooth group %s\n", yy1text );
  BEGIN( cSMOOTH_GROUP );
  PRINT( "state %d\n", yy_start );
  return( SMOOTH_GROUP );
}

{FLOATEXP} {
  PRINT( "float %s\n", yy1text );
  yy1lval.f = atof( yy1text );
  return( FLOAT );
}

{FLOAT}    { 
  //PRINT( "float %s\n", yy1text );
  yy1lval.f = atof( yy1text );
  return( FLOAT );
}

<*>{INTEGER}  { 
  PRINT( "integer %s\n", yy1text );
  yy1lval.i = atoi( yy1text );
  return( INTEGER );
}

{QUOTED_STRING} {
  char* theString = new char[yy1leng];
  char* outPtr = theString;

  char* inPtr = yy1text;
  inPtr++; /* Skip the first quote */
  
  while ( inPtr != '\0' )
  {
    if ( *inPtr == '"' )
    {
      if ( *( inPtr + 1 ) == '"' )
      {
        /* Replace double quotes by a single one */
        inPtr++;
      }
      else
      {
        /* A single quote ends the string */
        break;
      }
    }

    /* Copy the character to the output string */
    *outPtr = *inPtr;

    /* Go to the next character */
    outPtr++;
    inPtr++;
  }

  /* End the string */
  *outPtr = '\0';

  yy1lval.s = theString;
  return( STRING );
}

"/*"       {
multilinecomment:
  {
    char next;
    while ( ( ( next = yyinput() ) != '*' ) &&
            ( next != EOF ) )
    {
      /* Eat input. */
    }
    switch( yyinput() )
    {
      case '/':
        break; /* We found the comment end. */
      case EOF:
        printf( "Multi-line comment never ended.\n" );
        break; 
      case '*':
      {
        /* We need to put that one back in the input since it might be part of
        ** the end of a comment. */
        unput('*');
      }
      /* no break. */
      case '\n':
        //yy1lineno++;
        // Fall through.
      default:
        goto multilinecomment;
    }
  }
}

{SEPARATOR}      {
  PRINT( "separator %s\n", yy1text );
  return( SEPARATOR );
}

[#$][^\n]*\n {
  PRINT( "comment %s\n", yy1text );
  /* Skip one line comments. */
  //yy1lineno++;
}

<cGROUP,cMTLLIB>{SUPERID}        {
  PRINT( "superid %s\n", yy1text );
  yy1lval.s = yy1text;
  return( IDENTIFICATOR );
}

<*>{ID}             {
  PRINT( "id %s\n", yy1text );
  yy1lval.s = yy1text;
  return( IDENTIFICATOR );
}

<cFACE,cGROUP,cSMOOTH_GROUP,cMTLLIB>{NEWLINE} {
  //UpdateProgress( yy1lineno );
  PRINT( "cFACE newline %s\n", yy1text );
  BEGIN(INITIAL);
  return( NEWLINE );
}

<cFACE>{SPACE}   {
  PRINT( "cFACE space %s\n", yy1text );
  return( SPACE );
}

{NONWHITE}      {
  PRINT( "junk %s\n", yy1text );
  return( JUNK );
}

<*>{NEWLINE}        {
  PRINT( "newline %s\n", " " );
  //UpdateProgress( yy1lineno );
//yy1lineno++;
}

<*>{WHITE}          {
  PRINT( "state %d ", yy_start );
  PRINT( "plain white [%s]\n", yy1text );
}

%%
