/* Tokens */


%union
{
  int    i;
  double f;
  char*  s;
}


%{
#include "ff.h"
#include <string.h>
#include <stdio.h>
int yylex( void );
  
  enum TextureType gTextureType = kRandom;
  char* gLastID = 0;
  int gLastIDLen = 0;
  
  void yy1error( char* s );
%}


%token VERTEX
%token VERTEX_NORMAL
%token TEXTURE_COORD
%token MATERIAL
%token USE_MAP
%token USE_MATERIAL
%token GROUP
%token SMOOTH_GROUP
%token SHADOW_OBJECT
%token MAP_LIB
%token CINTERP
%token MATERIAL_LIB
%token RES

%token DISPLACEMENTMAP
%token RANDOM
%token SEPARATOR

%token TRIANGLE
%token QUADRILATERAL
%token FACE
%token LINE
%token BEZIER_PATCH
%token OBJECT

%token NEWLINE
%token SPACE

%token <f> FLOAT
%token <i> INTEGER

%token <s> STRING

%token IDENTIFICATOR

%token JUNK

%%

file : blocks
;

blocks : blocks block
|
; 

block : definitions objects
;

definitions : definitions definition
| definition
;

definition : def_vertex_xyz
| def_texture_uvw
| def_normal_xyz
| properties
| use_map
| use_material
| group
| smooth_group
| shadow_object
| mtl_lib
| map_lib
| c_interp
| res
| def_object
;

def_vertex_xyz : VERTEX vertex_coords
{
  AddVertex( gVertex );
  ResetVertexInfo();
}
;

vertex_coords : xyz xyz xyz
;

xyz : FLOAT
{
  gVertex.xyz[ gVertex.index ] = yy1lval.f;
  gVertex.index++;
}
;

xyz : INTEGER
{
  gVertex.xyz[ gVertex.index ] = yy1lval.i;
  gVertex.index++;
}
;

def_texture_uvw : TEXTURE_COORD uvw uvw uvw
{
  gFeatures.textures++;
  /* TBD:Eric Paquette: Implement if needed. */
  ResetTextureCoordinateInfo();
}
;

def_texture_uvw : TEXTURE_COORD uvw uvw
{
  gFeatures.textures++;
  AddTextureCoordinate2d( &gTextureCoord );
  ResetTextureCoordinateInfo();
}
;

uvw : FLOAT
{
  gTextureCoord.uvw[ gTextureCoord.index ] = yy1lval.f;
  gTextureCoord.index++;
}
;

uvw : INTEGER
{
  gTextureCoord.uvw[ gTextureCoord.index ] = yy1lval.i;
  gTextureCoord.index++;
}
;

properties : texture
;

objects : objects object
| object
;

object : triangle
| quadrilateral
| face
| line
| bezier_patch
;

triangle : TRIANGLE tri_indexes
{
  //BEGIN( INITIAL );
  //printf( "t state %d\n", yy_start );
  //printf( "finished face t\n" );
  gFeatures.faces++;
  AddFace( gVertices.xyzi, gVertices.index );
  ResetVerticesInfo();
}
;

tri_indexes : indexes indexes indexes NEWLINE
| indexes indexes indexes SPACE
;

quadrilateral : QUADRILATERAL quad_indexes
{
  //BEGIN( INITIAL );
  //printf( "q state %d\n", yy_start );
  //printf( "finished face q\n" );
  gFeatures.faces++;
  AddFace( gVertices.xyzi, gVertices.index );
  ResetVerticesInfo();
}
;

quad_indexes : indexes indexes indexes indexes NEWLINE
| indexes indexes indexes indexes SPACE
;

face : FACE face_indexes
{
  //BEGIN( INITIAL );
  //printf( "f state %d\n", yy_start );
  //printf( "finished face\n" );
  gFeatures.faces++;
  if ( gVertices.usedN && gVertices.usedUV ) 
    AddFace( gVertices.xyzi, gVertices.uvi, gVertices.ni, gVertices.index );
  else if ( !gVertices.usedN )
    AddFace( gVertices.xyzi, gVertices.index );
  else
    AddFace( gVertices.xyzi, gVertices.ni, gVertices.index );
  ResetVerticesInfo();
};

face_indexes : complex_index_list SPACE NEWLINE
| complex_index_list NEWLINE
| complex_index_list SPACE
;

complex_index_list : complex_index_list indexes
| indexes indexes
;

indexes : SPACE indexes_alone
;

indexes_alone : xyzi
| xyzi SEPARATOR uvi
| xyzi SEPARATOR uvi SEPARATOR
| xyzi SEPARATOR uvi SEPARATOR normali
| xyzi SEPARATOR SEPARATOR normali
;

xyzi : INTEGER
{
  if ( gVertices.index == gVertices.maxIndex )
  {
    /* Increment the amount of storage */
    int old_maxIndex = gVertices.maxIndex;
    int *old_xyzi = gVertices.xyzi;
    int *old_uvi = gVertices.uvi;
    int *old_ni = gVertices.ni;
    
    gVertices.maxIndex += 10;
    gVertices.xyzi = new int[gVertices.maxIndex];
    gVertices.uvi = new int[gVertices.maxIndex];
    gVertices.ni = new int[gVertices.maxIndex];

    for ( int i = 0 ; i < old_maxIndex ; i++ )
    {
      gVertices.xyzi[ i ] = old_xyzi[ i ];
      gVertices.uvi[ i ] = old_uvi[ i ];
      gVertices.ni[ i ] = old_ni[ i ];
    }

    delete[] old_xyzi;
    delete[] old_uvi;
    delete[] old_ni;
  }
  
  gVertices.xyzi[ gVertices.index ] = yy1lval.i;
  gVertices.index++;
}
;

uvi : INTEGER
{
  /* - 1 because the index is incremented in vertex_xyz which is called
     before */
  gVertices.usedUV = true;
  gVertices.uvi[ gVertices.index - 1 ] = yy1lval.i;
}
;

normali : INTEGER
{
  gVertices.usedN = true;
  gVertices.ni[ gVertices.index - 1 ] = yy1lval.i;
}
;

line : LINE simple_index_list
{
  gFeatures.lines++;
  /* TBD:Eric Paquette: Implement if needed */
}
;

bezier_patch : BEZIER_PATCH simple_index_list
{
  gFeatures.bezierPatches++;
  /* TBD:Eric Paquette: Implement if needed */
}
;

simple_index_list : simple_index_list simple_index
| simple_index
;

simple_index : simple_index_int SEPARATOR simple_index_int
| simple_index_int
;

simple_index_int : INTEGER
;

texture : displacement_map
;

displacement_map : MATERIAL DISPLACEMENTMAP texture_type
{
  AddDisplacementMap( /*gTextureType*/ );
}
;

texture_type : RANDOM
{
  gTextureType = kRandom;
}
;

use_map : USE_MAP ids
{
  gFeatures.maps++;
}
;

use_material : USE_MATERIAL ids
{
  gFeatures.materials++;
  gCurrentShader = GetShader( gLastID );
//  printf( "Material id {%s}\n", gLastID );
}
;


def_normal_xyz : VERTEX_NORMAL normal_coords
{
  gFeatures.normals++;
  AddNormal( gNormal );
  ResetNormalInfo();
}
;

normal_coords : nxyz nxyz nxyz
;

nxyz : FLOAT
{
  gNormal.xyz[ gNormal.index ] = yy1lval.f;
  gNormal.index++;
}
;

nxyz : INTEGER
{
  gNormal.xyz[ gNormal.index ] = yy1lval.i;
  gNormal.index++;
}
;


group : GROUP ids NEWLINE
{
  gFeatures.groups++;
  //BEGIN( INITIAL );
  //printf( "g state %d\n", yy_start );
  /* TBD:Eric Paquette: Implement if ever needed. */
}
;

smooth_group : SMOOTH_GROUP ids NEWLINE
{
  gFeatures.smoothGroups++;
  //BEGIN( INITIAL );
  //printf( "s state %d\n", yy_start );
  /* TBD:Eric Paquette: Implement if ever needed. */
}
;

ids : ids id
| ids INTEGER
|
;

id : IDENTIFICATOR
{
  if ( (size_t)gLastIDLen < strlen( yy1lval.s ) + 1 )
  {
    gLastIDLen = strlen( yy1lval.s ) + 1;
    delete[] gLastID;
    gLastID = new char[ gLastIDLen ];
  }
    
  strcpy( gLastID, yy1lval.s );
}
;

shadow_object : SHADOW_OBJECT ids
{
  gFeatures.shadowObjects++;
}
;

def_object : OBJECT ids
{
  /* TBD:Eric Paquette: Implement if needed */
}
;

map_lib : MAP_LIB ids
{
  gFeatures.mapLibs++;
}
;

c_interp : CINTERP ids
{
}
;

mtl_lib : MATERIAL_LIB filenames NEWLINE
{
  gFeatures.mtlLibs++;
  //BEGIN( INITIAL );
  //printf( "s state %d\n", yy_start );
}
;

filenames : filename
| filenames filename
;

filename : IDENTIFICATOR
{
  if ( !IsDefaultMaterialFile( yy1lval.s ) )
    LoadMaterialFile( yy1lval.s );
}
;

res : RES INTEGER INTEGER
{
  /* TBD:Eric Paquette: Implement if needed */
}
;

%%

#include "lex.yy1.c"

ffVertex gVertex;
ffNormal gNormal;

TextureCoord gTextureCoord;

Vertices gVertices;

Features gFeatures;


void yy1error( char* s )
{
  fprintf( stderr, "[ERROR] line %d : %s\n", yy1lineno, s );
  fprintf( stderr, "(%s)\n", yy1text );
}

//#if (!defined(__sgi))
char  yy1sbuf[40];
char* yy1sptr = yy1sbuf;
//#endif

bool yy1Init( const char* szFileName )
{
  //BEGIN( INITIAL );
  yy1in = fopen( szFileName, "r" );
  if ( yy1in == 0 )
  {
    fprintf( stderr, "[ERROR] yy1Init : could not open file [%s]\n",
             szFileName );
    return( false );
  }
  
  yy1lineno = 1;
  yy1leng = 0;
  yy1sbuf[0] = '\0';
  yy1sptr = yy1sbuf;
  
  return( true );
}

void yy1End()
{
  CloseFaceGroup();
  fclose( yy1in );
  yy1in = stdin;
}

