%{
#include <stdlib.h>
#include <stdio.h>
#if (defined(__sgi) && (_COMPILER_VERSION >= 721))
#pragma set woff 1110,1174,1516,1167,3322
#endif
#define PRINT(a,b)
//#define PRINT(a,b) printf( a, b )
%}

%option yylineno
%option noyywrap

%x cTEXTMAP

WHITE           [\t\n\ ]
NONWHITE        [^\t\n\ ]
NEWLINE         \n

SPACE           [\t\ ]+

DIGIT           [0-9]
FLOAT           {DIGIT}+|{DIGIT}+"."{DIGIT}*|"."{DIGIT}+

NEW_MATERIAL    newmtl
TEXTURE_MAP	map_Kd
AMBIENT         Ka
DIFFUSE         Kd
SPECULAR        Ks
SHININESS       Ns
NICOEFF         Ni
ILLUM_COEFF     illum
TEXTURECOEF     Tf

ALPHA           [a-zA-Z]
ALPHANUM        ({ALPHA}|[0-9])
ALPHAEXT        {ALPHANUM}|"-"|_|"."|"/"
ALPHAEXT2       {ALPHANUM}|"-"|:|_|"."|"/"
NID             {ALPHA}({ALPHAEXT2})*".tif"
ID              {ALPHA}({ALPHAEXT})*
SUPERID         {ALPHAEXT}({ALPHAEXT})*

%%

{NEW_MATERIAL}    {
  PRINT( "new material [%s]\n", yymtltext );
   BEGIN(INITIAL);
  return( NEW_MATERIAL );
};


{AMBIENT}         {
  PRINT( "ambient [%s]\n", yymtltext );
  return( AMBIENT );
};

{TEXTURE_MAP} 	{
  PRINT("Texture map [%s]\n", yymtltext );
//  BEGIN (cTEXTMAP );
  return TEXTURE_MAP;
};

{TEXTURECOEF}         {
  PRINT( "Texture Coeffs [%s]\n", yymtltext );
  return( TEXTURECOEF );
};

{DIFFUSE}         {
  PRINT( "diffuse [%s]\n", yymtltext );
  return( DIFFUSE );
};

{SPECULAR}        {
  PRINT( "specular [%s]\n", yymtltext );
  return( SPECULAR );
};

{SHININESS}       {
  PRINT( "shininess [%s]\n", yymtltext );
  return( SHININESS );
};

{ILLUM_COEFF}       {
  PRINT( "illum [%s]\n", yymtltext );
  return( ILLUM_COEFF );
};

{NICOEFF} {
  PRINT( "coef [%s]\n", yymtltext );
	return ( NICOEFF );
}
;
{FLOAT}    { 
  PRINT( "float [%s]\n", yymtltext );
  yymtllval.f = atof( yymtltext );
  return( FLOAT );
}
;


<*>{NID}             {
  PRINT( "nid [%s]\n", yymtltext );
  yymtllval.s = yymtltext;
//	BEGIN(INITIAL);
  return( NID );
}

<*>{ID}             {
  PRINT( "id [%s]\n", yymtltext );
  yymtllval.s = yymtltext;
	BEGIN(INITIAL);
  return( ID );
}


{SPACE}   {
  PRINT( "space [%s]\n", yymtltext );
  return( SPACE );
}

{WHITE}          {
  PRINT( "plain white [%s]\n", yymtltext );
  return( WHITE ); 
}

[#$][^\n]*\n {
  PRINT( "comment [%s]\n", yymtltext );
  /* Skip one line comments. */
  yymtllineno++;
}

  %%
