/* Tokens */


%union
{
  double f;
  char*  s;
}


%{
#include "ff.h"
#include <stdio.h>
#include <string.h>

int yylex( void );

  void yymtlerror( char* s );

  const int gMaterialNameSize = 256;
  char gMaterialName[ gMaterialNameSize ];
%}


%token NEW_MATERIAL

%token AMBIENT
%token DIFFUSE
%token SPECULAR
%token SHININESS
%token TEXTURECOEF
%token TEXTURE_MAP
%token ILLUM_COEFF
%token NICOEFF

%token SPACE
%token WHITE

%token <f> FLOAT

%token ID
%token NID
%token NEWLINE

%%

file : blocks
;

blocks : blocks block
| block
; 

block : newmtl attributes
{
  AddShader( gMaterialName );
}
;

newmtl : NEW_MATERIAL SPACE mtlname WHITE
;

mtlname : ID
{
  if ( strlen( yymtllval.s ) < gMaterialNameSize )
    strcpy( gMaterialName, yymtllval.s );
  else
    printf( "[ERROR] %s %d : name too long %s.\n",
            __FILE__, __LINE__, yymtllval.s );
//  fprintf(stderr, "New material %s\n", gMaterialName);
}

attributes : attributes attribute
| attribute
| illum
;

attribute : ambient
| texturecoeff
| diffuse
| specular
| shininess
| tmap
| nicoef
;

nicoef : NICOEFF SPACE fx WHITE
{
	gHasNiCoeff = true;
}
;

fx : FLOAT
{
	gNiCoeff = yymtllval.f;
} 
;

illum : ILLUM_COEFF SPACE fi WHITE
{
	gHasIllCoeff = true;
}
;

fi : FLOAT
{
	gIllCoeff = yymtllval.f;
}
;

texturecoeff : TEXTURECOEF SPACE tr SPACE tg SPACE tb WHITE
{
  gHasTextureCoef = true;
}
;

tr : FLOAT
{
  gTextureR = yymtllval.f;
}
;

tg : FLOAT
{
  gTextureG = yymtllval.f;
}
;

tb : FLOAT
{
  gTextureB = yymtllval.f;
}
;

tmap : TEXTURE_MAP SPACE filename WHITE
{
  gHasTextureMap = true;
}
;

filename : NID
{
printf("FILENAME %s\n", yymtllval.s);
  LoadTextureFile( yymtllval.s ); 
}
;

ambient : AMBIENT SPACE ar SPACE ag SPACE ab WHITE
{
  gHasAmbient = true;
}
;

ar : FLOAT
{
  gAmbientR = yymtllval.f;
}
;

ag : FLOAT
{
  gAmbientG = yymtllval.f;
}
;

ab : FLOAT
{
  gAmbientB = yymtllval.f;
}
;

diffuse : DIFFUSE SPACE dr SPACE dg SPACE db WHITE
{
  gHasDiffuse = true;
}
;

dr : FLOAT
{
  gDiffuseR = yymtllval.f;
}
;

dg : FLOAT
{
  gDiffuseG = yymtllval.f;
}
;

db : FLOAT
{
  gDiffuseB = yymtllval.f;
}
;

specular : SPECULAR SPACE sr SPACE sg SPACE sb WHITE
{
  gHasSpecular = true;
}
;

sr : FLOAT
{
  gSpecularR = yymtllval.f;
}
;

sg : FLOAT
{
  gSpecularG = yymtllval.f;
}
;

sb : FLOAT
{
  gSpecularB = yymtllval.f;
}
;

shininess : SHININESS SPACE n WHITE
{
  gHasSpecularN = true;
}
;


n : FLOAT
{
  gSpecularN = yymtllval.f;
}
;

%%

#include "lex.yymtl.c"

bool  gHasNiCoeff;
bool  gHasIllCoeff;
float gIllCoeff;
float gNiCoeff;

float gTextureR;
float gTextureG;
float gTextureB;
bool  gHasTextureCoef;

float gAmbientR;
float gAmbientG;
float gAmbientB;
bool  gHasAmbient;

bool  gHasTextureMap;

float gDiffuseR;
float gDiffuseG;
float gDiffuseB;
bool  gHasDiffuse;

float gSpecularR;
float gSpecularG;
float gSpecularB;
bool  gHasSpecular;

float gSpecularN;
bool  gHasSpecularN;

int gCurrentShader;

void yymtlerror( char* s )
{
  printf( "[ERROR] line %d : %s\n", yymtllineno, s );
  printf( "(%s)\n", yymtltext );
}

//#if (!defined(__sgi))
char  yymtlsbuf[40];
char* yymtlsptr = yymtlsbuf;
//#endif

bool yymtlInit( const char* szFileName )
{
  yymtlin = fopen( szFileName, "r" );
  if ( yymtlin == 0 )
    return( false );
  yymtllineno = 1;
  yymtlleng = 0;
  yymtlsbuf[0] = '\0';
  yymtlsptr = yymtlsbuf;
  return( true );
}

void yymtlEnd()
{
  fclose( yymtlin );
  yymtlin = stdin;
}

